@@ -4,10 +4,11 @@ from pkg_resources import DistributionNotFound, get_distribution
 
 from .binaryconfusionmatrix import BinaryConfusionMatrix  # noqa: F401
 from .confusionmatrix import ConfusionMatrix  # noqa: F401
-from .constants import *
+from .constants import *  # noqa: F401, F403
 from .data_methods import *  # noqa: F401, F403
 from .display import print_grid, print_image  # noqa: F401
 from .file_methods import *  # noqa: F401, F403
+from .general import *  # noqa: F401, F403
 from .goudapath import GoudaPath  # noqa: F401
 from .moving_stats import *  # noqa: F401, F403
 
