@@ -157,6 +157,8 @@ def print_grid(*images, figsize=(8, 8), toFile=None, show=True, return_grid_shap
     if show:  # pragma: no cover
         # Check manually
         plt.show()
+    else:
+        plt.close(fig)
     if return_grid_shape:
         return rows, cols
 
@@ -240,9 +242,14 @@ def squarify(image, axis=0, as_array=False):
         num_images = len(image)
         images = [item for item in image]
     else:
-        images = np.split(image, image.shape[axis], axis=axis)
-        images = [item for item in images]
+        # images = np.split(image, image.shape[axis], axis=axis)
+        # images = [item for item in images]
+        images = []
+        axis_slice = [slice(None) for _ in range(image.ndim)]
         num_images = image.shape[axis]
+        for idx in range(num_images):
+            axis_slice[axis] = idx
+            images.append(image[tuple(axis_slice)])
     num_rows = int(np.ceil(np.sqrt(num_images)))
     outer_list = []
     for i in range(0, num_images, num_rows):
